import { getLogger } from './loggers';
import { doSearch } from './twitter';

const logger = getLogger('command');

function parseCmd(message: string): {
  cmd: string;
  args: string[];
} {
  message = message.trim();
  message = message.replace('\\\\', '\\0x5c');
  message = message.replace('\\\"', '\\0x22');
  message = message.replace('\\\'', '\\0x27');
  const strs = message.match(/'[\s\S]*?'|(?:\S+=)?"[\s\S]*?"|\S+/mg);
  const cmd = strs?.length ? strs[0].length ? strs[0].substring(0, 1) === '/' ? strs[0].substring(1) : '' : '' : '';
  const args = (strs ?? []).slice(1).map(arg => {
    arg = arg.replace(/^(\S+=)?["']+(?!.*=)|["']+$/g, '$1');
    arg = arg.replace('\\0x27', '\\\'');
    arg = arg.replace('\\0x22', '\\\"');
    arg = arg.replace('\\0x5c', '\\\\');
    return arg;
  });
  return {
    cmd,
    args,
  };
}

function search(chat: IChat, args: string[], reply: (msg: string) => any): void {
  if (args.length === 0 || !args[0]) {
    return reply('未输入自定义查询表达式。');
  }
  const conf: Parameters<typeof doSearch>[0] = {query: args[0]};
  const confZH: Record<Exclude<keyof typeof conf, 'query'>, string> = {
    count: '数量上限',
    since: '起始点',
    until: '结束点',
  };
  for (const arg of args.slice(1)) {
    const optMatch = /^(count|since|until)=(.*)/.exec(arg);
    if (!optMatch) return reply(`未定义的查询参数：${arg}。`);
    const optKey = optMatch[1] as keyof typeof confZH;
    if (optMatch.length === 1) return reply(`查询${confZH[optKey]}参数格式有误。`);
    conf[optKey] = optMatch[2];
    if (optMatch[2] === '') return reply(`查询${confZH[optKey]}参数值不可为空。`);
  }
  if (conf.count !== undefined && !Number(conf.count) || Number(conf.count) > 100) {
    return reply('查询数量上限参数非正、非数值或超出取值范围。');
  }
  try {
    doSearch(conf, chat);
  } catch (e) {
    logger.error(`error querying timeline, error: ${e}`);
    reply('推特机器人尚未加载完毕，请稍后重试。');
  }
}

function picSearch(chat: IChat, args: string[], reply: (msg: string) => any): void {
  search(chat, [args[0] && args[0] + '(filter:media)', ...args.slice(1)], reply);
}
export { parseCmd, search, picSearch };
